import re
import numpy as np
import warnings
import math
import xarray as xr

from DDRW import setcolors

#######################################################
def check_variable(data,varname,args):
    if not varname in data.variables:
        print("variable ", varname, " is not found")
        exit(1)

    if not hasattr(args,"WIND"):
        return
    
    # check if U and V are included in NETCDF
    if args.WIND is not None:
        if not ("U" in data.variables and "V" in data.variables):
            print("U and V are not found in NETCDF")
            exit(1)
        if int(args.WIND) != 0 and int(args.WIND) != 1:
            print("-W 0 or 1: 0->wind vector 1-> wind barb")
            exit(1)

        if not hasattr(args,"X0_VCS"):
            return
            
        #### check if vertical cross section or not
        if args.X0_VCS is not None:
            if int(args.WIND) !=0:
                print("wind vector only accepted in vertical cross section")
                exit(1)
            if not ("W" in data.variables or "OMEGA" in data.variables):
                print("any vertical information is not available")
                exit(1)

            if "OMEGA" in data.variables:
                print("only horizontal wind is shown in vertical cross section")

#######################################################
def get_TOPOdata(data,varname,args):
    
    if check_TOPO_in_variable(data,varname):
        TOPO = data[varname]
        return TOPO
    else:
        topofile = args.TOPO
        topodata = xr.open_dataset(topofile)
        TOPO = topodata[varname]
        return TOPO
    
#######################################################
def check_TOPO_in_variable(data,varname):
    if varname in data.variables:
        print("variable ", varname, " is found")
        return 1
    else:
        return 0
#######################################################
def set_dim_name(data):

    dimnameT = "NOTDEFINED"
    dimnameZ = "NOTDEFINED"
    dimnameY = "NOTDEFINED"
    dimnameX = "NOTDEFINED"
    for ddd in data.dims:
        if ddd == "TIME":
            dimnameT = "TIME"
        elif ddd == "time":
            dimnameT = "time"
        elif ddd == "Time":
            dimnameT = "Time"
        elif ddd == "LEV":
            dimnameZ = "LEV"
        elif ddd == "ALT":
            dimnameZ = "ALT"
        elif ddd == "LAT":
            dimnameY = "LAT"
        elif ddd == "LON":
            dimnameX = "LON"
    if dimnameT == "NOTDEFINED":
        print("TIME DIMENSION NAME is not found")
        exit(1)
    if dimnameZ == "NOTDEFINED":
        print("Z DIMENSION NAME is not found")
        exit(1)
    if dimnameY == "NOTDEFINED":
        print("Y DIMENSION NAME is not found")
        exit(1)
    if dimnameX == "NOTDEFINED":
        print("X DIMENSION NAME is not found")
        exit(1)
    return dimnameT,dimnameZ,dimnameY,dimnameX

#######################################################
def get3D(args,dimnameT,data,varname):
    
    if args.timenum is None and args.TIME is None:
        print("chose your time from following list \n")
        for i, a in enumerate(data[dimnameT].values):
            print(i,a)
        print("enter TNUM:")
        t = int(input())
        TTT = data[dimnameT].values[t]
        timesel={dimnameT : t}
        V3D = data[varname].isel(timesel)

        if args.WIND is not None:
            windU3D = data["U"].isel(timesel)
            windV3D = data["V"].isel(timesel)
    elif args.timenum is None:
        print("YYYY-MM-DD_HH:MN:SC format \n")
        ptn = r"\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d"
        if not re.match(ptn,args.TIME):
            print("TIME INPUT FORMAT WRONG")
            print("Y4-M2-D2TH2:M2:S2")
            print("ex 2020-07-06T03:00:00")
            exit(1)
        TTT = args.TIME
        timesel={dimnameT : args.TIME}
        V3D = data[varname].sel(timesel)
        if args.WIND is not None:
            windU3D = data["U"].sel(timesel)
            windV3D = data["V"].sel(timesel)
    else:
        t = int(args.timenum)
        timesel={dimnameT : t}
        TTT = data[dimnameT].values[t]
        V3D = data[varname].isel(timesel)
        if args.WIND is not None:
            windU3D = data["U"].isel(timesel)
            windV3D = data["V"].isel(timesel)

    if args.WIND is not None:
        return V3D,TTT,windU3D,windV3D
    else:
        return V3D,TTT

#######################################################
def get2D(args,dimnameZ,data,V3D,windU3D="None",windV3D="None"):
    
    if args.levelnum is None and args.LEV is None:
        print("chose your lev from following list \n")
        for k, a in enumerate(data[dimnameZ].values):
            print(k,a)
        print("enter ZNUM:")
        k = int(input())
        ZZZ = data[dimnameZ].values[k]
        levsel={dimnameZ : k}
        V2D = V3D.isel(levsel)
        if args.WIND is not None:
            windU2D = windU3D.isel(levsel)
            windV2D = windV3D.isel(levsel)

    elif args.levelnum is None:
        ZZZ = args.LEV
        levsel={dimnameZ : ZZZ}
        V2D = V3D.sel(levsel)
        if args.WIND is not None:
            windU2D = windU3D.sel(levsel)
            windV2D = windV3D.sel(levsel)
    else:
        k = int(args.levelnum)
        levsel={dimnameZ : k}
        ZZZ = data[dimnameZ].values[k]
        V2D = V3D.isel(levsel)
        if args.WIND is not None:
            windU2D = windU3D.isel(levsel)
            windV2D = windV3D.isel(levsel)

    if args.WIND is not None:
        return V2D,ZZZ,windU2D,windV2D
    else:
        return V2D,ZZZ

#######################################################
def check_draw_domain(slon,elon,slat,elat,data):

    #############################################################
    # check the dimension name    
    dimnameT,dimnameZ,dimnameY,dimnameX = set_dim_name(data)
    #############################################################
    
    NY = data[dimnameY].size
    NX = data[dimnameX].size
    if slat < data[dimnameY].values[0] or \
       elat > data[dimnameY].values[NY-1] or \
       slon < data[dimnameX].values[0] or \
       elon > data[dimnameX].values[NX-1] :
        print("drawing domain is outside of data")
        print("LAT=", data[dimnameY].values[0],\
              " - ", data[dimnameY].values[NY-1])
        print("LON=", data[dimnameX].values[0],\
              " - ", data[dimnameX].values[NX-1])
        exit(1)

#######################################################
def set_color_range(args,V2D):

    if args.ColorType is not None:
        if int(args.ColorType) == 0:
            colormap = setcolors.set_Blue2Red(0)
        elif int(args.ColorType) == 1:
            colormap = setcolors.set_jma_rain(0)
        elif int(args.ColorType) == 2:
            colormap = setcolors.set_Blue2White(0)
        else:
            print("set colormap for variable")
            exit(1)
    else:
        print ("at present, user cannot manually set for color pallet")
        exit(1)

    #############################################################
    # automatically set variable range for color shade 
    if args.AutoRange is None:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore",category=RuntimeWarning)
            maxV = np.nanmax(list(map(lambda x: np.nanmax(x), V2D.values)))
            minV = np.nanmin(list(map(lambda x: np.nanmin(x), V2D.values)))
        numcolor = colormap.N+1

        if int(maxV) > int(minV):
            maxV = int(maxV)
            minV = int(minV)
        else:
            pass
        clevels = np.linspace(minV,maxV,numcolor)
    else:
        # user set manually max - min range for color shade
        minV = 10.0
        maxV = 61.0
        numcolor = colormap.N+1
        #clevels = np.linspace(minV,maxV,numcolor)
        steps = int((maxV-minV)/numcolor)
        clevels = np.arange(minV,maxV,steps)

    return colormap,clevels

#############################################################
def optimize_wind_vector_length(windU2D,windV2D):

    with warnings.catch_warnings():
        warnings.simplefilter("ignore",category=RuntimeWarning)
        maxUU = np.nanmax(list(map(lambda x: np.nanmax(x), windU2D.values)))
        minUU = np.nanmin(list(map(lambda x: np.nanmin(x), windU2D.values)))
        maxVV = np.nanmax(list(map(lambda x: np.nanmax(x), windV2D.values)))
        minVV = np.nanmin(list(map(lambda x: np.nanmin(x), windV2D.values)))
    maxUV = max(maxUU,maxVV)
    minUV = max(minUU,minVV)
    difUV = (maxUV-minUV)/5
    difUV = np.round(difUV,1)
    maxUV = maxUV*1.01
    minUV = minUV - abs(minUV)*0.01
    maxUV = np.round(maxUV,1)
    minUV = np.round(minUV,1)
    maxUV = max(maxUV,abs(minUV))
    maxUV = np.round(maxUV,0)
    # draw arrow with "1-inch" length -> scaleUV m/s
    # 1 inch = 2.54 cm 
    scaleUV = maxUV * 2.54 * 2.0
    # maxwind will be shown by arrow with length of 0.5 cm
    #print("maxwind=",maxUV)
    return scaleUV,maxUV
###############################################################
def lon_per1km(LAT):

    meanLAT = np.mean(LAT)
    dlon = 1.e0/111.111 / math.cos(math.radians(meanLAT))
    return dlon

def set_ticks(slon,elon,slat,elat):

    if elon -slon > 1.0:
        dlon = int(((elon-slon)/4.0)*10.0)/10.0
    else:
        dlon = (elon-slon)/5.0

    if elat -slat > 1.0:
        dlat = int(((elat-slat)/4.0)*10.0)/10.0
    else:
        dlat = (elat-slat)/5.0
        
    xticks0 = np.arange(slon,elon+0.01,dlon)
    yticks0 = np.arange(slat,elat+0.01,dlat)

    if dlon >= 0.1:
        xticks = np.round(xticks0,1)
    else:
        xticks = np.round(xticks0,2)
    if dlat >= 0.1:
        yticks = np.round(yticks0,1)
    else:
        yticks = np.round(yticks0,2)
        
    return (xticks,yticks)
